@@ -22,21 +22,44 @@ if TYPE_CHECKING:
 
     from typing_extensions import Self
 
-    from narwhals._compliant.typing import EvalNames
     from narwhals._expression_parsing import ExprKind, ExprMetadata
-    from narwhals._polars.dataframe import Method, PolarsDataFrame
+    from narwhals._polars.dataframe import Method
     from narwhals._polars.namespace import PolarsNamespace
-    from narwhals._utils import Version, _LimitedContext
+    from narwhals._utils import Version
     from narwhals.typing import IntoDType, ModeKeepStrategy, NumericLiteral
 
 
 class PolarsExpr:
-    _implementation = Implementation.POLARS
+    # CompliantExpr
+    _implementation: Implementation = Implementation.POLARS
+    _version: Version
+    _native_expr: pl.Expr
+    _metadata: ExprMetadata | None = None
+    _evaluate_output_names: Any
+    _alias_output_names: Any
+    __call__: Any
+
+    # CompliantExpr + builtin descriptor
+    # TODO @dangotbanned: Remove in #2713
+    @classmethod
+    def from_column_names(cls, *_: Any, **__: Any) -> Self:
+        raise NotImplementedError
+
+    @classmethod
+    def from_column_indices(cls, *_: Any, **__: Any) -> Self:
+        raise NotImplementedError
+
+    def __narwhals_expr__(self) -> Self:  # pragma: no cover
+        return self
+
+    def __narwhals_namespace__(self) -> PolarsNamespace:  # pragma: no cover
+        from narwhals._polars.namespace import PolarsNamespace
+
+        return PolarsNamespace(version=self._version)
 
     def __init__(self, expr: pl.Expr, version: Version) -> None:
         self._native_expr = expr
         self._version = version
-        self._metadata: ExprMetadata | None = None
 
     @property
     def _backend_version(self) -> tuple[int, ...]:
@@ -52,10 +75,6 @@ class PolarsExpr:
     def _with_native(self, expr: pl.Expr) -> Self:
         return self.__class__(expr, self._version)
 
-    @classmethod
-    def _from_series(cls, series: Any) -> Self:
-        return cls(series.native, series._version)
-
     def broadcast(self, kind: Literal[ExprKind.AGGREGATION, ExprKind.LITERAL]) -> Self:
         # Let Polars do its thing.
         return self
@@ -234,12 +253,6 @@ class PolarsExpr:
     def cum_count(self, *, reverse: bool) -> Self:
         return self._with_native(self.native.cum_count(reverse=reverse))
 
-    def __narwhals_expr__(self) -> None: ...
-    def __narwhals_namespace__(self) -> PolarsNamespace:  # pragma: no cover
-        from narwhals._polars.namespace import PolarsNamespace
-
-        return PolarsNamespace(version=self._version)
-
     def is_close(
         self,
         other: Self | NumericLiteral,
@@ -308,33 +321,6 @@ class PolarsExpr:
     def struct(self) -> PolarsExprStructNamespace:
         return PolarsExprStructNamespace(self)
 
-    # CompliantExpr
-    _alias_output_names: Any
-    _evaluate_aliases: Any
-    _evaluate_output_names: Any
-    _is_multi_output_unnamed: Any
-    __call__: Any
-
-    # CompliantExpr + builtin descriptor
-    # TODO @dangotbanned: Remove in #2713
-    @classmethod
-    def from_column_names(
-        cls,
-        evaluate_column_names: EvalNames[PolarsDataFrame],
-        /,
-        *,
-        context: _LimitedContext,
-    ) -> Self:
-        raise NotImplementedError
-
-    @classmethod
-    def from_column_indices(cls, *column_indices: int, context: _LimitedContext) -> Self:
-        raise NotImplementedError
-
-    @staticmethod
-    def _eval_names_indices(indices: Sequence[int], /) -> EvalNames[PolarsDataFrame]:
-        raise NotImplementedError
-
     # Polars
     abs: Method[Self]
     all: Method[Self]
